/* Parallel Hierarchical Grid -- an adaptive finite element library.
 *
 * Copyright (C) 2005-2007 State Key Laboratory of Scientific and
 * Engineering Computing, Chinese Academy of Sciences. */

/* This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA */

/* $Id: dof-utils.h,v 1.16 2007/06/21 08:49:55 zlb Exp $ */

#ifndef PHG_DOF_UTILS_H

/* function prototypes */

#ifdef __cplusplus
extern "C" {
#endif

FLOAT *phgDofEval_(DOF *dof, SIMPLEX *e, const FLOAT lambda[],
			const FLOAT *basvalues, int bas_stride,
			FLOAT *values, FLOAT **data);
#define phgDofEval(dof, e, lambda, values) \
	phgDofEval_(dof, e, lambda, NULL, 0, values, NULL)

FLOAT *phgDofEvalGradient(DOF *dof, SIMPLEX *e, const FLOAT lambda[],
			const FLOAT *gradbas, FLOAT *values);
#define phgDofGradient(src, dest, newtype, name) \
	phgDofGradient_(src, dest, newtype, name, __FILE__, __LINE__)
DOF *phgDofGradient_(DOF *src, DOF **newdof, DOF_TYPE *newtype,
			const char *name, const char *srcfile, int srcline);

FLOAT *phgDofEvalDivergence(DOF *dof, SIMPLEX *e, const FLOAT lambda[],
			const FLOAT *gradbas, FLOAT *values);
#define phgDofDivergence(src, dest, newtype, name) \
	phgDofDivergence_(src, dest, newtype, name, __FILE__, __LINE__)
DOF *phgDofDivergence_(DOF *src, DOF **newdof, DOF_TYPE *newtype,
			const char *name, const char *srcfile, int srcline);
FLOAT *phgDofEvalCurl(DOF *dof, SIMPLEX *e, const FLOAT lambda[],
			const FLOAT *gradbas, FLOAT *values);
#define phgDofCurl(src, dest, newtype, name) \
	phgDofCurl_(src, dest, newtype, name, __FILE__, __LINE__)
DOF *phgDofCurl_(DOF *src, DOF **newdof, DOF_TYPE *newtype,
			const char *name, const char *srcfile, int srcline);

/* Utility functions */
BOOLEAN phgDofDirichletBC(DOF *u, SIMPLEX *e, int dof_index,
		DOF_USER_FUNC func, FLOAT mat[], FLOAT rhs[], DOF_PROJ proj);

/* BLAS-like functions */
#define phgDofCopy(src, dest, newtype, name) \
	phgDofCopy_(src, dest, newtype, name, __FILE__, __LINE__)
DOF *phgDofCopy_(DOF *src, DOF **dest, DOF_TYPE *newtype,
		 const char *name, const char *srcfile, int srcline);

FLOAT phgDofNormL2Vec(DOF *x);
FLOAT phgDofNormL1Vec(DOF *x);
FLOAT phgDofNormInftyVec(DOF *x);

FLOAT phgDofNormL1(DOF *u);
FLOAT phgDofNormL2(DOF *u);
FLOAT phgDofNormH1(DOF *u);

void phgDofRandomize(DOF *u, long int seed);

#define phgDofMM(transa, transb, M, N, K, alpha, A, blka, B, beta, C) \
	phgDofMM_(transa, transb, M, N, K, alpha, A, blka, B, beta, C, \
		  __FILE__, __LINE__)
DOF *phgDofMM_(MAT_OP transa, MAT_OP transb, int M, int N, int K,
          FLOAT alpha, DOF *A, int blka, DOF *B, FLOAT beta, DOF **Cptr,
          const char *srcfile, int srcline);

DOF *phgDofAXPBY_(FLOAT a, DOF *x, FLOAT b, DOF **y,
				const char *srcfile, int srcline);
#define phgDofAXPBY(a, x, b, y) \
	phgDofAXPBY_(a, x, b, y, __FILE__, __LINE__)
#define phgDofAXPY(a, x, y)	phgDofAXPBY(a, x, (FLOAT)1.L, y)

DOF *phgDofMatVec_(FLOAT alpha, DOF *A, DOF *x, FLOAT beta, DOF **y_ptr,
		const char *srcfile, int srcline);
#define phgDofMatVec(alpha, A, x, beta, y) \
	phgDofMatVec_(alpha, A, x, beta, y, __FILE__, __LINE__)

#ifdef __cplusplus
}
#endif

#define PHG_DOF_UTILS_H
#endif
